#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  gen_config.py
#     run to create the config.py file in current directory
#
#     this should only be run to initialize the config.py file
#
#  Copyright 2024 cswaim <cswaim@jcrl.net>
#  Licensed under the Apache License, Version 2.0
#  http://www.apache.org/licenses/LICENSE-2.0

import os
import config_tpg as ac

def run():
    """ copy the cofig file to the current dir"""

    """
    Generates a skeleton config.py file in the specified directory.

    Args:
        output_dir (str): The directory to generate the config file in.
    """
    # Get the directory of the package
    package_dir = os.path.dirname(ac.__file__)

    mod_list = ['config.py', 'configparms_ext.py']

    for m in mod_list:
        # Full path to the template file
        src = os.path.join(package_dir, m)
        dst_dir = os.getcwd()

        # Destination path in the current directory
        dst = os.path.join(dst_dir, m)

        if os.path.exists(dst):
            print("")
            print(f"    ** {m} already exists in {dst_dir} **")
            print("        Skipping generation.")
            continue

        # Check the operating system and use the respective command
        if os.name == 'nt':  # Windows
            cmd = f'copy "{src}" "{dst}"'
        else:  # Unix/Linux
            cmd = f'cp "{src}" "{dst}"'

        print("")
        print(f"    {m} generated successfully in {dst_dir}")
        # Copy File
        os.system(cmd)

    print("")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  config_ext_mod.py
#     in each module, from src import config as cfg
#     access variables in this module as cfg.xxxxxx
#
#     the config file is created in the data directory and can be modified
#     for a specific run.
#
#  Copyright 2024 cswaim <cswaim@jcrl.net>
#  Licensed under the Apache License, Version 2.0
#  http://www.apache.org/licenses/LICENSE-2.0

# import this module as the first application module:
#     import src.config as cfg

# system variables change only the cfg_flnm
cfg_flnm = 'test_config.cfg'
wkdir_path = None
wkdir = None
srcdir = None
datadir = None

# variables
var1 = True
var2 = 2
var3 = 3.4
m1 ='textm1'
m2 = ['m2-1','m2-2','m2-3']
seed = None

# system variables
sys_cfg_version = '0.1'
sys_var = "sysvar test variable"

# values passed to ConfigParms
# dict key is the section, value is list of variable names and type
#   types are i-integer, f-float, b-boolean, s-string, l-list

cfg_values = {'MAIN': [('var1', 'b'), ('var2', 'i'), ('var3', 'f')],
              'DATA': [('m1', 's'), ('m2', 'l'), ('seed', 'f')],
              'SYSTEM': [('sys_cfg_version', 's'), ('sys_var', 's')],
              }
cfg_comments = {'sys_cfg_version': ['changing the version number will cause file to be rewritten'],
                'sys_var': ['sys var cmt1', 'sys var cmt2'],
                'var1': ['this is a comment for var1'],
                'm2': ['m2 comment 1', 'm2 comment 2'],
                'DATA': ['sec comment 1', 'sec comment 2']
                }

# config obj
config = None
# variables passed to all modules
gen_var1 = []
mod='config_ext_mod'

"""
This module takes advantage of Python's behavior of importing the module
the first time and for every import after the first, only a reference is passed.  The code is not re-executed.

There are several ways to instantiate the ConfigParm class which reads the
cfg file.  Pick an approach that you like.

Note that setting the autorun may effect tests as the defaults from the
config file are loaded and the test defaults must be reset.

to autorun on the first import:
    cp = ConfigParms(cfg_values, cfg_comments, autorun=True)

to control the run in your application, just instantiate the class in this module
    run_init()

and then in the application code, read the parm file:
    cfg.run()
"""

# the imports must be at end of the config module
#
# To override the behavior of ConfigParms
# such as change the path to data or scr directories
# or to modify the default behavior for a section or variable
# (1) make the modifications in the configparms_ext modult
#
# the init will look for the configparms_ext module first and use it
# if it exists, otherwise it will use the package configparms module

def run_init():
    """run the init"""
    global cp, cu
    try:
        from src.configparms_ext import ConfigParmsExt as ConfigParms
    except Exception as e:
        from config_tpg.configparms import ConfigParms

    from config_tpg.configutils import ConfigUtils
    cp = ConfigParms(cfg_values, cfg_comments)
    cu = ConfigUtils()

def run():
    """read the config file"""
    cp.run()   # to load the config file & set values in module

# instantiate the configparms module
run_init()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  test_config.py
#
#  Copyright 2023 cswaim <cswaim@jcrl.net>
#  Licensed under the Apache License, Version 2.0
#  http://www.apache.org/licenses/LICENSE-2.0

import unittest
import importlib
import os
import sys
from src import setup_module

class TestConfig(unittest.TestCase):
    """ tests for the """

    # testfile_path = f"tests{os.sep}testfiles{os.sep}"
    testfile_path = f"testfiles{os.sep}"

    @classmethod
    def setUpClass(cls):
        """class set up"""
        print("\n ------- \nTesting module - test_config.py")
        if not os.path.exists(cls.testfile_path):
           os.mkdir(cls.testfile_path)

        # copy ext_mod to ext
        global cfg
        setup_module.run()
        # import the ext to overide imports in cfg
        from src import config as cfg
        sys.modules['src.configparms_ext'] = importlib.reload(sys.modules['src.configparms_ext'])
        importlib.reload(cfg)

        cfg.datadir = cls.testfile_path
        cfg.run()
        return

    @classmethod
    def tearDownClass(cls):
        """class tear down"""
        if os.path.exists(cls.testfile_path):
            for pth, dir, files in os.walk(cls.testfile_path):
                for fl in files:
                     os.remove(f"{cls.testfile_path}{fl}")
            os.rmdir(cls.testfile_path)
        return

    def setUp(self):

        return

    def tearDown(self):

        return

    def test_update(self):

        pass

    def test_default_config(self,):
        """test set_default_config """

        # load the default values
        config = cfg.cp.read_config_file(cfg.config)
        # When experienting with different config value,
        #    might not pass.
        self.assertTrue(cfg.var1)
        self.assertEqual(cfg.m1, 'textm1')


    def test_adding_new_data_item(self,):
        """test add new data item  """
        def prt_file(data_dir, flnm):
            with open(f"{data_dir}{flnm}", 'r') as cf:
               fdata = cf.read()
            print(fdata)

        # load the default values
        config = cfg.cp.read_config_file(cfg.config)
        orig_version = config.get("SYSTEM", "sys_cfg_version")
        # print("initial default ini file")
        prt_file(cfg.datadir, cfg.cfg_flnm)

        # remove items from config & change version
        new_version = "0.0.0"
        config.remove_option('MAIN', 'var2')
        config.remove_option("SYSTEM", "sys_var")
        config.set("SYSTEM","sys_cfg_version", new_version)
        cfg.cp.write_cfg(config)
        config = cfg.cp.set_config_module_variables(config)
        # confirm  var2 has been removed
        self.assertFalse(config.has_option("MAIN", "var2"))
        self.assertFalse(config.has_option("SYSTEM", "sys_var"))

        # read and build missing options
        config = cfg.cp.read_config_file(cfg.config)
        self.assertEqual(orig_version, cfg.sys_cfg_version)
        self.assertTrue(config.has_option("MAIN", "var2"))
        self.assertTrue(config.has_option("SYSTEM", "sys_var"))

    def test_remove_default_comments(self,):
        """test remove_default_comments """
        # load the default values
        config = cfg.cp.set_default_config(cfg.config)
        comments = dict((k, v) for k, v in config['DATA'].items() if k.startswith('#'))
        cfg.cp.remove_default_comments(cfg.config)

        for k in comments.keys():
            self.assertFalse(config.has_option("DATA", k))

    def test_print_config_vars(self,):
        """test print of config vars """
        # load the default values
        print("  before removal of comments")
        cfg.cu.print_config_vars(heading='test of print::no fileobj')
        config = cfg.cp.set_default_config(cfg.config)
        cfg.cp.remove_default_comments(cfg.config)
        cfg.cu.print_config_vars(heading='test of print::no fileobj')
        # print without comments
        cfg.cu.print_config_vars(heading='test of print::no comments', comments=False)

if __name__ == '__main__':
    # unittest.main()

    cf = unittest.TestLoader().loadTestsFromTestCase(TestConfig)
    unittest.TextTestRunner(verbosity=2).run(cf)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  untitled.py
#
#  Copyright 2024 cswaim <cswaim@tpginc.net>

import src.config as cfg

class ConfigUtils():
    """ a class of utilities for the config module """

    @staticmethod
    def print_config_vars( heading=None, comments=True, fileobj=None ):
        print("", file=fileobj)
        if heading is not None:
            print(f"--- {heading} ---", file=fileobj)

        print(f"    wkdir: {cfg.wkdir}", file=fileobj)
        print(f"  wk path: {cfg.wkdir_path}", file=fileobj)
        print(f"  src dir: {cfg.srcdir}", file=fileobj)
        print(f" data dir: {cfg.datadir}", file=fileobj)
        print(f"file name: {cfg.cfg_flnm}", file=fileobj)
        print("", file=fileobj)

        print(f"sections: {cfg.config.sections()}", file=fileobj)

        # print config variables
        for sec, vars in cfg.config.items():
            if comments:
                ConfigUtils.print_cfg_var_comments(sec, sec=True, fileobj=fileobj)
            print(cfg.config[sec], file=fileobj)
            for var, val in vars.items():
                if comments:
                    ConfigUtils.print_cfg_var_comments(var, fileobj=fileobj)
                print(f"   {var}: {val}", file=fileobj)

    @staticmethod
    def print_cfg_var_comments( var, sec=False, fileobj=None):
        """look for comments for the var (sec or var)"""
        if var in cfg.cfg_comments.keys():
                for c in cfg.cfg_comments[var]:
                    if sec:
                        print(f"# {c}", file=fileobj)
                    else:
                        print(f"   # {c}", file=fileobj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  test_setup_module.py
#
#  Copyright 2024 cswaim <cswaim@tpginc.net>

import os
import sys
import config_tpg as ac

def run():
    """ this will set up the src folder within the test folder
        A new copy of the config and configparms_ext are copied to
        test/src.  This assures tests are run with the most recent
        copies of this members.

        The confg module can be modified by subsequent tests, but the
        changes are not kept because this module will refresh the config
        with copies from the package.
        """

    print("===== in the test setup module =====")
    package_dir = os.path.dirname(ac.__file__)

    mod_list = ['config.py', 'configparms_ext.py']

    for m in mod_list:
        # Full path to the template file
        src = os.path.join(package_dir, m)

        # Destination path in the test.src directory
        dst = os.path.dirname(__file__)

        # Check the operating system and use the respective command
        if os.name == 'nt':  # Windows
            cmd = f'copy "{src}" "{dst}"'
        else:  # Unix/Linux
            cmd = f'cp "{src}" "{dst}"'

        try:
            # Copy File
            os.system(cmd)
            print("")
            print(f"    {m} generated successfully in {dst}")
        except Exception as e:
            print(e)

    print("===== exit the test setup module =====")

def run_ext():
    """ copy the config_ext_mod.py to config.py

        the modified config overrides the configparms
        """

    print("===== in the test ext setup module =====")

    # a list of package files  (src file, dest file)
    mod_list = [ ('config_ext_mod.py', 'config.py'),
                 ('configparms_ext_mod.py', 'configparms_ext.py')]

    for f in mod_list:
        s, d = f

        src = os.path.join(os.path.dirname(__file__), s)
        dst = os.path.join(os.path.dirname(__file__), d)

        """copy the src file to the dst file"""
        # Check the operating system and use the respective command
        if os.name == 'nt':  # Windows
            cmd = f'copy "{src}" "{dst}"'
        else:  # Unix/Linux
            cmd = f'cp "{src}" "{dst}"'

        try:
            # Copy File
            os.system(cmd)
            print("")
            print(f"    {s} copied successfully to {dst}")
        except Exception as e:
            print(e)

    print("===== exit the test ext setup module =====")


if __name__ == '__main__':
    run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  test_configparms_ext.py
#
#  Copyright 2023 cswaim <cswaim@jcrl.net>
#  Licensed under the Apache License, Version 2.0
#  http://www.apache.org/licenses/LICENSE-2.0

import unittest
import importlib
import os
import sys
from src import setup_module

class TestConfigParmsExt(unittest.TestCase):
    """ tests for the """

    # testfile_path = f"tests{os.sep}data{os.sep}"
    testfile_path = f"testfiles{os.sep}"

    @classmethod
    def setUpClass(cls):
        """class set up"""
        print("\n ------- \nTesting module - test_configparms_ext.py")
        if not os.path.exists(cls.testfile_path):
           os.mkdir(cls.testfile_path)

        # copy ext_mod to ext
        global cfg
        setup_module.run_ext()
        # import the ext to overide imports in cfg
        from src import config as cfg
        sys.modules['src.configparms_ext'] = importlib.reload(sys.modules['src.configparms_ext'])
        importlib.reload(cfg)

        # load the cfg with default paths
        # cp is instantiated in the config module
        cfg.datadir = cls.testfile_path
        cfg.run()

        return

    @classmethod
    def tearDownClass(cls):
        """class tear down"""
        dir_list = [cls.testfile_path,]

        for i in dir_list:
            if os.path.exists(i):
                for pth, dir, files in os.walk(i):
                    for fl in files:
                        os.remove(f"{i}{fl}")
                os.rmdir(i)
        return

    def setUp(self):

        return

    def tearDown(self):

        return

    def test_update(self):

        pass

    def test_set_directories(self,):
        """test set directories """

        # data directory set in ext
        self.assertEqual(cfg.datadir, self.testfile_path)

        # value added in init routine set_directories
        self.assertEqual(cfg.extdir, "added in set_directories")

    def test_custom_init_routine(self,):
        """test custom init routine"""
        self.assertEqual(cfg.init_var, 'loaded from ext')

    # test setting values in config object

    def test_set_custom_default_sects(self,):
        """ test set custom default sects """
        self.assertEqual(cfg.scds, "from set_custom_default_sects")

    def test_set_custom_default_vars(self,):
        """ test set custom default vars """
        self.assertEqual(cfg.scdv, "from set_custom_default_vars")

    # test setting the values in the cfg module

    def test_set_module_sects(self):
        """ test set module sects """
        self.assertEqual(cfg.sms, "from set_module_sects")

    def test_set_module_vars(self):
        """ test set module vars """
        self.assertEqual(cfg.smv, "from set_module_vars")
        self.assertEqual(cfg.seed, .023)

    def test_set_custom_module_vars(self):
        """ test set custiom module vars """
        self.assertEqual(cfg.scmv, "from set_custom_module_vars")

    # test verifying the values in the config parser object

    def test_verify_config_sects(self):
        """ test verify config sects """
        self.assertEqual(cfg.vcs, "from verify_config_sects")

    def test_verify_config_vars(self):
        """ test verify config vars """
        self.assertEqual(cfg.vcv, "from verify_config_vars")

if __name__ == '__main__':

    cf = unittest.TestLoader().loadTestsFromTestCase(TestConfigParmsExt)
    unittest.TextTestRunner(verbosity=2).run(cf)
